/*******************************************************************************

   Tiny_Clock

   Relgio com mostrador LCD 16x2 (usando mtodo Lplex)

   Autor: Cludio Lrios

   Data: 09/01/2017
   
   Procedimento para ajuste do relgio usando um nico boto (interruptor):
   
   Aperte o boto de ajuste (ADJ) at que comece a piscar o dgito de dezena de 
   horas. Solte o boto logo que comece a piscar. Aperte e solte o boto para
   incrementar este primeiro dgito. Apertar e soltar rpidamente no resultar
   em incrementos. Faa de forma mais lenta, para que a rotina possa operar
   corretamente.
   Quando atingir o valor desejado, aperte o boto e mantenha pressionado at
   que comece a piscar o prximo dgito. Repita o ajuste deste dgito. Faa isto
   com todos os dgitos. Caso no deseje alterar um dgito, poder manter o
   boto pressionado at que passe para o prximo dgito. Caso necessite ajustar
   apenas alguns dgitos iniciais como ex. horas, poder ajustar e sar do
   ajuste simplesmente aguardando sem apertar o boto. Depois de 5 segundos
   ir sair do modo de ajuste automaticamente, parando de piscar e comeando a
   incrementar os dgitos de segundos no modo normal de trabalho.

   IMPORTANTE: Use capacitor de 220 nF junto aos pinos 1 e 8 do PIC
   pois o circuito  sensvel a interferncias pela alimentao.
   Use outro capacitor de 220 nF do vcc para o gnd do LCD, bem prximo
   destes terminais.

   Uso didtico
   Este arquivo  parte integrante do blog Lrios.Tecnologia.ws
*******************************************************************************/
#include <12F675.h>
#device adc=8
#use delay(clock=4000000)//use xtal externo de 4mhz nos pinos 2  e 3 do pic
#fuses NOWDT,HS, NOCPD, NOPROTECT, NOMCLR, NOPUT, NOBROWNOUT
#zero_ram
//byte
#byte   TRISIO      = 0x85
#byte   GPIO        = 0x05
#byte   TMR1H       = 0x0F
#byte   TMR1L       = 0x0E
#byte   TMR0        = 0X01
#byte   OPTION_REG  = 0x81
#byte   WPU         = 0x95
#byte   CMCON       = 0x19
#byte   ANSEL       = 0x9F
#byte   T1CON       = 0X10
#byte   IOC         = 0x96
//bit
#bit    GIE         = 0xb.7
#bit    PEIE        = 0xb.6
#bit    T0IE        = 0xb.5
#bit    TMR1IE      = 0x8C.0
#bit    TMR1IF      = 0xC.0
#bit    T0IF        = 0XB.2


//pinos do lcd (metodo Lariosplex - 3 fios)
#bit DB76  = 0x5.0  //pino 7 db7=direto db6=resistor 10k +cap 10nF ao gnd
#bit DB54  = 0x5.1  //pino 6 db5=direto db4=resistor 10k +cap 10nF ao gnd
#bit E_RS  = 0x5.2  //pino 5 E=direto RS=resistor 10k +cap 10nF ao gnd
#bit sw    = 0x5.3  //pino 4 boto de ajuste de horas/minutos


//defines
#define vcarga 300 // por tentativa e erro (entre 200 a 700)
#define vdescarga 500//idem
#define v_delay_clk 2  // de 2 a 50 conforme LCD
#define _XTAL  4000000  //frequencia do cristal usado
#define presc  64 // valor do prescaller do timer usado (8bits)
#define fator  (256*presc*4)
#define VALOR_PISCA  15  //estava  15
#define VALOR_SW_AP  10  //10
#define VALOR_ADJ    10 //10
#define VALOR_OUT_AUTO 25 //


//variveis globais
unsigned int1 RS;//flag usado no envio de comandos/dados para lcd
unsigned int1 f_pisca;//flag para funcionamento das piscadas ao ajustar relgio
unsigned int1 f_tr1;//flag de trava em operaes de ajuste do relgio
unsigned int1 f_tr2;//idem
unsigned int8 dez_hora;//armazena dezenas de horas
unsigned int8 un_hora;//armazena unidade de horas
unsigned int8 dez_min;//armazena dezenas de minutos
unsigned int8 un_min;//armazena unidade de minutos
unsigned int8 dez_seg;//armazena dezenas de segundos
unsigned int8 un_seg;//armazena unidade de minutos
unsigned int8 cont;//contador usado na temporizao das piscadas ao ajustar
unsigned int8 vez;//armazena qual dgito ser feito o ajuste
unsigned int8 cont_p;//conta a pausa inicial apertando o boto p/ entrar em modo de ajuste
unsigned int8 cont_auto;//conta o tempo para sada do ajuste de forma automtica 
unsigned int32 cnt32=0;//armazena acmulo do tempo para obter um segundo

/*==============================================================================
       ROTINA DE INTERRUPO DO TIMER 0 (OBTEM TEMPO DE 1 SEGUNDO)
==============================================================================*/
#INT_TIMER0
void  isr_tmr_0(){
   //T0IF=0;//reseta flag over flow do timer 0
   if(++cont==VALOR_PISCA){cont=0; f_pisca=!f_pisca;}
   if(vez!=0)return;

   cnt32+=fator;
   if(cnt32>=_XTAL){
        cnt32-=_XTAL;
        if(++un_seg>9) {
           un_seg=0;
           if(++dez_seg>5){
             dez_seg=0;
             if(++un_min>9){
              un_min=0;
              if(++dez_min>5){
                dez_min=0;
                if(++un_hora>9){
                   un_hora=0;
                   dez_hora++;
                }//if(++un_hora>9)
              }//if(++dez_min>59)
             }//if(++un_min>9)
           }//if(++dez_seg>59)
        }//if(++un_seg>9)
         if((dez_hora==2)&&(un_hora>3)){dez_hora=0; un_hora=0;}
     }// if(cnt32>=XTAL)
  }//void  isr_tmr_0()
/*******************************************************************************
                          SUBROTINAS
*******************************************************************************/


void descarregar(){ DB76=0;DB54=0;E_RS=0; delay_us(vdescarga);}
void t_carga(){delay_us(vcarga);}
void clk(){ E_RS=1; delay_cycles(v_delay_clk); E_RS=0;}//gera clock par Lcd


/*******************************************************************************
                          SUBROTINAS de LCD
*******************************************************************************/


/*******************************************************************************
                         ENVIA NIBBLE
*******************************************************************************/
void env_nible(unsigned int8 nib){
    disable_interrupts(int_timer0);
   // disable_interrupts(int_ra);
    descarregar();//descarrega capacitores
    nib&=0x0f;//limita
    if(nib&0x01)DB54=1;//testa bit 0
    if(nib&0x04)DB76=1;//testa bit 2
    if(RS)E_RS=1;else E_RS=0;//comando ou dado
    t_carga();
    if(nib&0x02)DB54=1; else DB54=0;//testa bit 1
    if(nib&0x08)DB76=1; else DB76=0;//testa bit 3
    clk();//clock
    enable_interrupts(int_timer0);

}

/*******************************************************************************
                          ENVIA DADOS PARA O LCD
*******************************************************************************/

void env_lcd_d(unsigned int8 dado){

 RS=1;//indica que enviar 'dados'
 env_nible(dado>>4);//high nibble primeiro
 env_nible(dado);
 delay_ms(4);
}

/*******************************************************************************
                         ENVIA COMANDOS PARA LCD
*******************************************************************************/
void env_lcd_c(unsigned int8 comando){

 RS=0;//indica que enviar 'comandos'
 env_nible(comando>>4);//high nibble primeiro
 env_nible(comando);
 delay_ms(4);
}

/*******************************************************************************
                         INICIALIZAO DO LCD
*******************************************************************************/

void Lcd_Init(){
  unsigned int8 a;

   descarregar();//descarrega capacitores
   RS=0; //enviar comando
   delay_ms(40);//delay inicial
   for(a=0;a<3;a++) { env_nible(3);delay_ms(5);}//envia 3x 0x03
   env_nible(2);//envia 0x02
   delay_ms(5);//delay 5 ms
   env_lcd_c(0x28);//ajusta para 4 bits o modo de envio
   env_lcd_c(0x06);// no shift
   env_lcd_c(0x0c);//Display ON, Cursor OFF, Blink OFF
   env_lcd_c(0x01);//limpa tela
}


void mostra_lcd(){

      env_lcd_c(0x80);//linha 1
      printf(env_lcd_d,"   Tiny Clock  ");//mostra o nome
      env_lcd_c(0xc0);//linha 2
      printf(env_lcd_d,"    ");//apaga antes
      if((vez==1)&&f_pisca){env_lcd_d(' ');}//espao
      else{env_lcd_d(dez_hora|'0');}//envia dezena de horas
      if((vez==2)&&f_pisca){env_lcd_d(' ');}//espao
      else{env_lcd_d(un_hora|'0');}//envia unidade de horas
      env_lcd_d(':');//envia 2 pontos
      if((vez==3)&&f_pisca){env_lcd_d(' ');}//espao
      else{env_lcd_d(dez_min|'0');}//envia dezena de minutos
      if((vez==4)&&f_pisca){env_lcd_d(' ');}//espao
      else{env_lcd_d(un_min|'0');}//envia unidade de minutos
      env_lcd_d(':');//envia 2 pontos
      if((vez==5)&&f_pisca){env_lcd_d(' ');}//espao
      else{env_lcd_d(dez_seg|'0');}//envia dezena de segundos
      if((vez==6)&&f_pisca){env_lcd_d(' ');}//espao
      else{env_lcd_d(un_seg|'0');}//envia unidade de segundos
      printf(env_lcd_d,"    ");//apaga depois
}
/*******************************************************************************
            ROTINA PARA AJUSTE DE HORAS/MINUTOS/SEGUNDOS
*******************************************************************************/
//Entrando em modo de ajuste do relgio:
//Aperte o boto por 3 segundos at comear a piscar o primeiro dgito msb
void testa_adj(){ 

      if(!sw){//escolhe digito sequencialmente por aperto acima de 2 segundos
        f_tr1=1;
        if(++cont_p>VALOR_SW_AP){
           cont_p=0;
           cont_auto=0;//zera tempo de sada automtica do ajuste
           if(++vez>6)vez=0;
           f_tr2=1;
         }
      }
     
// Ajustando cada dgito:
// Aperte o boto e solte 
// A cada aperto ocorrer incremento do dgito que est piscando
    if(sw && f_tr1 && !f_tr2 && (cont_p<VALOR_SW_AP)){

        switch (vez){
        case 1:{if(++dez_hora>2)dez_hora=0; break;}//incrementa dezena de horas
        case 2:{ if(dez_hora<2){if(++un_hora>9)un_hora=0; break;}//acima de 19h
                 else{if(++un_hora>3)un_hora=0; break;}//abaixo de 20 horas
               }
        case 3:{if(++dez_min>5)dez_min=0; break;}//incrementa dezena de minutos
        case 4:{if(++un_min>9)un_min=0; break;}//incrementa unidade de minutos
        case 5:{if(++dez_seg>5)dez_seg=0; break;}//incrementa dezena de segundos
        case 6:{if(++un_seg>9)un_seg=0; break;}//incrementa unidade de segundos
        }
        cont_p=0;//zera contador
        f_tr1=0;//reseta flag
        cont_auto=0;//zera contador
      }
      f_tr2=0;//reseta flag trava 2
      if(vez)++cont_auto;// sada do modo de ajuste de forma automtica
      if(cont_auto>VALOR_OUT_AUTO){vez=0;cont_p=0; cont_auto=0;}//idem
}
/*******************************************************************************
                    ROTINA MAIN
*******************************************************************************/
void main() {

   setup_adc_ports(NO_ANALOGS|VSS_VDD);//libera ports para digital
   setup_adc(ADC_OFF);//desliga ADC
   setup_counters(RTCC_INTERNAL,RTCC_DIV_64);//tmr0  prescaller/64
   setup_timer_1(T1_DISABLED);//desliga timer 1
   setup_comparator(NC_NC_NC_NC);//desliga comparadores internos
   setup_vref(FALSE);//desliga voltagem de referncia
   trisio=0b111000;//ajusta entradas e sadas
   gpio=0;//zera tudo
   descarregar();//descarga dos capacitores do Lariosplex
   Lcd_Init();//inicializa Lcd
   printf(env_lcd_d,"   Tiny Clock   ");//mostra o nome
   env_lcd_c(0xc0);//linha 2
   printf(env_lcd_d,"Larios.Tecnol.Ws");// o blog
   delay_ms(2000);
   env_lcd_c(1);//apaga tela
   enable_interrupts(int_timer0);//habilita interrupo do timer 0
   enable_interrupts(global);//habilita interrupes
   f_pisca=0;//reseta flags e contadores
   f_tr1=0;//idem
   f_tr2=0;//idem
   vez=0;//idem
   cont_p=0;//idem


/*******************************************************************************
                    LAO PRINCIPAL DE REPETIO
*******************************************************************************/
   for(;;){
      testa_adj();//testa se deve realizar ajustes no relgio pelo usurio
      mostra_lcd();//mostra no lcd
      delay_ms(50);//delay entre apresentaes no lcd 
     }//for(;;)

}//main()
